@@ -1,4 +1,5 @@
 #include <RcppArmadillo.h>
+
 #if defined(_OPENMP)
 #include <omp.h>
 // [[Rcpp::plugins(openmp)]]
@@ -6,56 +7,53 @@
 
 using namespace arma;
 
-// [[Rcpp::export]]
+const int iMaxLag = 20;
 
+// [[Rcpp::export]]
 arma::vec HACWeightC(int iLag){ 
+  // Input will always be non-zero and positive, thus, the 0 case is ignored.
   vec vW = linspace(1, iLag, iLag) / iLag;
   int iIdx = sum(vW<=0.5);
   vW(span(0,iIdx-1)) = 1-6*square(vW(span(0,iIdx-1))) + 6 * pow(vW(span(0,iIdx-1)), 3);
   vW(span(iIdx, iLag-1)) = 2 * pow(1 - vW(span(iIdx, iLag-1)), 3);
   return(vW);
-  // Input will always be non-zero and positive, thus, the 0 case is ignored.
 }
 
 
 //[[Rcpp::export]]
 double AsymptoticVarianceC(arma::vec vIn, int iLag){
-  try{
-    double d0 = sum(square(vIn));
-    if(iLag == 0){
-      vec vAc = zeros(1);
+    try{
+      double d0 = sum(vIn % vIn);
+      vec vAc = zeros(iLag);
+      if(iLag == 0){
+        vec vAc = zeros(1);
+      }
+      int iT = vIn.size();
+      for(int i=0; i<iLag; i++){
+        vAc[i] = sum(vIn(span((i+1), iT-1)) % vIn(span(0, (iT-i-2))));
+      }
+      vec vW = HACWeightC(iLag);
+      double dOut = d0 + 2.0 * sum(vAc % vW);
+      return(dOut);
     }
-    vec vAc = zeros(iLag);
-    int iT = vIn.size();
-    for(int i=0; i<iLag; i++){
-      
-      vAc[i] = sum(vIn(span((i+1), iT-1)) % vIn(span(0, (iT-i-2))));
+    catch( std::exception &ex ) {
+      return(datum::nan);
+    } catch(...) {
+      return(datum::nan);
     }
-    vec vW = HACWeightC(iLag);
-    double dOut = d0 + 2.0 * sum(vAc % vW);
-    return(dOut);
-  }
-  catch( std::exception &ex ) {
-    return(-1.0);
-  } catch(...) {
-    return(-1.0);
-  }
-  
 }
 
+
 // [[Rcpp::export]]
 int AutomaticLagSelectionC(arma::vec vX, double dMu){
-  
   double dC = 2.6614;
   
   int iN = round(4.0 * pow((dMu/100), 0.16));
   
-  
   int iT = vX.size();
   vec vE = zeros(iN+1);
   vec vAc = zeros(iN);
   
-  
   for(int i=0; i<(iN+1); i++){
     vE[i] = mean(vX(span((i+1), iT-1)) % vX(span(0, (iT-i-2))));
   }
@@ -73,6 +71,7 @@ int AutomaticLagSelectionC(arma::vec vX, double dMu){
   double dGamma = dC * pow(pow((sQ / s0), 2.0 ), 0.2);
   
   int iOut = round(dGamma * pow(dMu, 0.2));
+  
   return(iOut);
   
   
@@ -87,44 +86,29 @@ Rcpp::List DriftBurstLoopC(arma::vec vPreAveraged, arma::vec diffedlogprices, ar
   int iQ;
   int iIdx;
   int iAutoAcLag;
-  double dReturnINF = 0.0;
-  double dFooMu;
-  
-  
-  
   arma::vec vMu = zeros<vec>(iT);
   arma::vec vSigma = zeros<vec>(iT);
-  arma::vec vX = zeros<vec>(iN);
-  arma::vec vWm = zeros<vec>(iN);
-  arma::vec vWvar = zeros<vec>(iN);
-  int iMaxLag = 20;
+  arma::vec vX;
+  arma::vec vWm;
+  arma::vec vWvar;
+  arma::vec vFoo(2);
+  vFoo[1] = iMaxLag;
   for(int i=1; i<iT; i++){
     
     vX = vTime(span(0, (iN-1))) - vTesttime[i];
-    
     iIdx = sum((vX<=0))-1;
     
     vWm = exp(-abs(vX(span(0,iIdx))/iMeanBandwidth));  // exponential kernel
-    dFooMu = sum(vWm);
-    
-    
+
     vMu[i] = sum(vWm(span(0,iIdx)) % vPreAveraged(span(0,iIdx))) / iMeanBandwidth; 
-    
+
     vWvar = exp(-abs(vX(span(0,iIdx))/iVarBandwidth));  // exponential kernel
-    
-    
-    
+
     if(iAcLag == -1){
-      iQ = AutomaticLagSelectionC(diffedlogprices, dFooMu);
-      vec vFoo(2); vFoo[0] = iQ ; vFoo[1] = iMaxLag;
-      int iAutoAcLag = min(vFoo) + 2.0 * (iPreAverage-1);
-      
-      
+      iQ = AutomaticLagSelectionC(diffedlogprices, sum(vWm));
+      vFoo[0] = iQ+ 2.0 * (iPreAverage-1);
+      iAutoAcLag = min(vFoo);
       double dAsympVar = AsymptoticVarianceC((vWvar(span(0,iIdx)) % vPreAveraged(span(0,iIdx))), iAutoAcLag) / iVarBandwidth; 
-      
-      if(dAsympVar == -1.0){
-        dReturnINF = 1;
-      }
       vSigma[i] = dAsympVar;
       
     }else{
@@ -134,12 +118,7 @@ Rcpp::List DriftBurstLoopC(arma::vec vPreAveraged, arma::vec diffedlogprices, ar
     
     
   }
-  
-  if(dReturnINF == 1){
-    arma::vec vFoo(1);
-    vFoo[0] = R_PosInf;
-    return(Rcpp::List::create(Rcpp::Named("DriftBursts") = vFoo));
-  }
+
   arma::vec vDb = sqrt(iMeanBandwidth) * vMu / sqrt(vSigma) ; 
   vDb[0] = 0;
   return(Rcpp::List::create(Rcpp::Named("DriftBursts") = vDb,
@@ -162,24 +141,19 @@ Rcpp::List DriftBurstLoopCPAR(arma::vec vPreAveraged, arma::vec diffedlogprices,
   omp_set_num_threads(iCores);
   int iT = vTesttime.size();
   int iN = vPreAveraged.size();
-  double dFooMu;
   int iIdx;
-  
-  arma::vec vMu= zeros<vec>(iT);
-  
-  arma::vec vSigma= zeros<vec>(iT);
-  
-  arma::vec vX= zeros<vec>(iN);
-  arma::vec vWm = zeros<vec>(iN);
-  arma::vec vWvar = zeros<vec>(iN);
-  
-  double dReturnINF = 0.0;
   int i;
-  int iMaxLag = 20;
+  arma::vec vMu= zeros<vec>(iT);
+  arma::vec vSigma= zeros<vec>(iT);
+  arma::vec vX;
+  arma::vec vWm;
+  arma::vec vWvar;
+  arma::vec vFoo(2); 
+  vFoo[1] = iMaxLag;
   //Parallelization setup
-#pragma omp parallel for default(none)                                                                                                                                   \
-  shared(vPreAveraged, diffedlogprices, vTime, vTesttime, iMeanBandwidth, iVarBandwidth, iPreAverage, iAcLag, vMu, vSigma, iT, iMaxLag, dReturnINF, iN) \
-    private(vX, vWm, vWvar, i, iIdx, dFooMu)
+#pragma omp parallel for default(none)\
+  shared(vPreAveraged, diffedlogprices, vTime, vTesttime, iMeanBandwidth, iVarBandwidth, iPreAverage, iAcLag, vMu, vSigma, iT, iN, vFoo) \
+    private(vX, vWm, vWvar, i, iIdx)
     
     for(i=1; i<iT; i++){
       
@@ -188,8 +162,6 @@ Rcpp::List DriftBurstLoopCPAR(arma::vec vPreAveraged, arma::vec diffedlogprices,
       iIdx = sum((vX<=0))-1;
       
       vWm = exp(-abs(vX(span(0,iIdx))/iMeanBandwidth));  // exponential kernel
-      dFooMu = sum(vWm);
-      
       
       vMu[i] = sum(vWm(span(0,iIdx)) % vPreAveraged(span(0,iIdx))) / iMeanBandwidth; 
       
@@ -198,15 +170,11 @@ Rcpp::List DriftBurstLoopCPAR(arma::vec vPreAveraged, arma::vec diffedlogprices,
       
       
       if(iAcLag == -1){
-        int iQ = AutomaticLagSelectionC(diffedlogprices, dFooMu);
-        arma::vec vFoo(2); vFoo[0] = iQ ; vFoo[1] = iMaxLag;
-        int iAutoAcLag = min(vFoo) + 2.0 * (iPreAverage-1);
         
+        int iQ = AutomaticLagSelectionC(diffedlogprices, sum(vWm));
+        vFoo[0] = iQ + 2.0 * (iPreAverage-1); 
+        int iAutoAcLag = min(vFoo);
         double dAsympVar = AsymptoticVarianceC((vWvar(span(0,iIdx)) % vPreAveraged(span(0,iIdx))),iAutoAcLag) / iVarBandwidth; 
-        
-        if(dAsympVar == -1.0){
-          dReturnINF = 1;
-        }
         vSigma[i] = dAsympVar;
         
       }else{
@@ -217,7 +185,8 @@ Rcpp::List DriftBurstLoopCPAR(arma::vec vPreAveraged, arma::vec diffedlogprices,
       
       
     }
-    arma::vec vDb = sqrt(iMeanBandwidth) * vMu / sqrt(vSigma) ; 
+    
+  arma::vec vDb = sqrt(iMeanBandwidth) * vMu / sqrt(vSigma) ; 
   Rcpp::List lOut =  Rcpp::List::create(Rcpp::Named("DriftBursts") = vDb,
                                         Rcpp::Named("Sigma") = vSigma,
                                         Rcpp::Named("Mu") = vMu);
@@ -225,11 +194,6 @@ Rcpp::List DriftBurstLoopCPAR(arma::vec vPreAveraged, arma::vec diffedlogprices,
   Rf_warning("OpenMP is not available. Sequential processing is used.");
   lOut = DriftBurstLoopC(vPreAveraged, diffedlogprices, vTime , vTesttime, iMeanBandwidth, iVarBandwidth, iPreAverage, iAcLag )
 #endif
-    if(dReturnINF == 1){
-      arma::vec vFoo(1);
-      vFoo[0] = R_PosInf;
-      return(Rcpp::List::create(Rcpp::Named("Failed") = vFoo));
-    }
     
     return(lOut);
     
@@ -237,3 +201,5 @@ Rcpp::List DriftBurstLoopCPAR(arma::vec vPreAveraged, arma::vec diffedlogprices,
 }
  
 
+
+ 
